mod header;
mod page;

use std::{fs::File, io::{Read, Seek}};
use bytes::Buf;

use header::SqliteHeader;
use page::{varint_read_from_buf, BTreePage, Value};

fn main() -> anyhow::Result<()> {
    let mut f = File::options().read(true).write(true).open("db.sqlite")?;
    let header = SqliteHeader::read_from_reader(& mut f)?;
    println!("{:?}", header);
    // print page size
    println!("Page size:  {}", header.page_size);
    
    let mut buf = vec![0; header.page_size as usize];
    f.seek(std::io::SeekFrom::Start(0))?;
    f.read_exact(&mut buf)?;
    let btr = BTreePage::read(&buf, true)?;
    // println!("{:?}", btr);
    let mut table_count = 0;
    match btr {
        BTreePage::LeafNode(leaf) => {
            println!("{:?}", leaf);
            leaf.data_cells.iter().for_each(|cell| {
                let buf = &cell.data;
                let mut buf = buf.as_slice();
                
                let (mut column_size, i) = varint_read_from_buf(&mut buf);
                let mut serial_types = vec![];
                column_size -= i as u64;
                while column_size > 0 {
                    let (serial_type, i) = varint_read_from_buf(&mut buf);
                    serial_types.push(serial_type);
                    column_size -= i as u64;
                }
                let values = serial_types.iter().map(|&serial_type| {
                    let v = Value::from_bytes(&mut buf, serial_type);
                    println!("{:?}", v);
                    v
                }).collect::<Vec<_>>();
                println!("{:?}", values);
                if let Some(v) = values.first() {
                    if let Value::Text(s) = v {
                        if s == "table" {
                            table_count += 1;
                        }
                    }
                }
            });
        }
        BTreePage::InternalNode(internal) => {
            println!("{:?}", internal);
        }
    }
    println!("Table count: {}", table_count);
    // get 
    Ok(())
}
